@@ -89,6 +89,9 @@ struct super_block *afs_read_super(struct super_block *sb, void *data,
     sb->s_blocksize_bits = 10;
     sb->s_magic = AFS_VFSMAGIC;
     sb->s_op = &afs_sops;	/* Super block (vfs) ops */
+#if defined(MAX_NON_LFS)
+    sb->s_maxbytes = MAX_NON_LFS;
+#endif
     code = afs_root(sb);
     if (code)
 	MOD_DEC_USE_COUNT;
@@ -134,7 +137,7 @@ static int afs_root(struct super_block *afsp)
 		/* setup super_block and mount point inode. */
 		afs_globalVp = tvp;
 #if defined(AFS_LINUX24_ENV)
-		afsp->s_root = d_alloc_root((struct inode*)tvp);
+		afsp->s_root = d_alloc_root((struct inode*)&tvp->v);
 #else
 		afsp->s_root = d_alloc_root((struct inode*)tvp, NULL);
 #endif
@@ -210,7 +213,11 @@ static LIST_HEAD(dummy_inode_list);
  * pages to disk. So it needs an inode syncing function to update metadata when it
  * has synced some pages of a file to disk.
  */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
+void afs_write_inode(struct inode *ip, int unused) 
+#else
 void afs_write_inode(struct inode *ip) 
+#endif
 {
     /* and put it back on our dummy list. */
     list_del(&ip->i_list);
@@ -227,37 +234,19 @@ void afs_write_inode(struct inode *ip)
  * If we use the common inode pool, we'll need to set i_nlink to 0 here.
  * That will trigger the call to delete routine.
  */
+
 void afs_delete_inode(struct inode *ip)
 {
-    cred_t *credp = crref();
     struct vcache *vc = (struct vcache*)ip;
 
     AFS_GLOCK();
-#if defined(AFS_LINUX24_ENV)
-    lock_kernel();
-    if (atomic_read(&ip->i_count) > 1)
-#else
-    if (ip->i_count > 1)
-#endif
-	printf("afs_put_inode: ino %d (0x%x) has count %d\n", ip->i_ino, ip);
-
     ObtainWriteLock(&vc->lock, 504);
-    afs_InactiveVCache(vc, credp);
-#if defined(AFS_LINUX24_ENV)
-    atomic_set(&ip->i_count, 0);
-#else
-    ip->i_count = 0;
-#endif
-    ip->i_nlink = 0; /* iput checks this after calling this routine. */
+    osi_clear_inode(ip);
     ReleaseWriteLock(&vc->lock);
-
-#ifdef AFS_LINUX24_ENV
-    unlock_kernel();
-#endif
     AFS_GUNLOCK();
-    crfree(credp);
 }
 
+
 /* afs_put_super
  * Called from unmount to release super_block. */
 void afs_put_super(struct super_block *sbp)
@@ -269,8 +258,10 @@ void afs_put_super(struct super_block *sbp)
     AFS_GLOCK();
     AFS_STATCNT(afs_unmount);
 
-    if (!suser())
+    if (!suser()) {
+	AFS_GUNLOCK();
 	return;
+    }
 
     afs_globalVFS = 0;
     afs_globalVp = 0;
