@@ -702,7 +702,7 @@ NB:  Has to be unsigned, since shifts on signed quantities may preserve
    clients.  This is probably OK, but I don't want to see too much of it.
 */
 
-#define	ranstage(x)	(x)= (afs_uint32) (3141592621*((afs_uint32)x)+1)
+#define	ranstage(x)	(x)= (afs_uint32) (3141592621U*((afs_uint32)x)+1)
 extern afs_int32 rxi_getaddr();
 
 unsigned int afs_random()
@@ -1202,6 +1202,13 @@ static afs_SetServerPrefs(sa)
 #ifdef AFS_SGI62_ENV
     (void) hash_enum(&hashinfo_inaddr, afsi_enum_set_rank, HTF_INET, NULL,
 		     (caddr_t)sa, NULL);
+#elif defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+    {
+        struct in_ifaddr *ifa;
+        TAILQ_FOREACH(ifa , &in_ifaddrhead, ia_link) {
+            afsi_SetServerIPRank(sa, ifa);
+        }
+    }
 #else
     {
 	extern struct in_ifaddr *in_ifaddr;
