@@ -169,7 +169,7 @@ int DumpUser (
     if ((!ka_KeyIsZero((char *) &tentry.key, sizeof(tentry.key))) && 
 	(showkey)) {
 	printf ("  key (%d):", tentry.key_version);
-	ka_PrintBytes (&tentry.key, sizeof(tentry.key));
+	ka_PrintBytes ((char *)&tentry.key, sizeof(tentry.key));
     }
     else {
 	if (tentry.keyCheckSum == 0) 
@@ -704,6 +704,13 @@ int StringToKey (
 
     printf ("Converting %s in realm '%s' yields key='",
 	    as->parms[0].items->data, realm);
+    ka_PrintBytes ((char *)&key, sizeof(key));
+    printf ("'.\n");
+
+    des_string_to_key (as->parms[0].items->data, &key);
+      
+    printf ("Converting %s with the DES string to key yields key='",
+	    as->parms[0].items->data);
     ka_PrintBytes (&key, sizeof(key));
     printf ("'.\n");
 
@@ -745,9 +752,10 @@ int SetPassword (
       ka_StringToKey (as->parms[1].items->data, realm, &key);
     } 
     else if (as->parms[2].items) {
-      if (ka_ReadBytes (as->parms[2].items->data, &key, sizeof(key)) != 8) {
+      if (ka_ReadBytes (as->parms[2].items->data, (char *)&key, sizeof(key))
+	  != 8) {
 	printf ("Key must be 8 bytes: '%s' was too long\n",
-			 as->parms[2].items->data);
+		as->parms[2].items->data);
 	return KABADCMD;
       }
     } 
@@ -890,10 +898,10 @@ static afs_int32 ListTicket (
     }
     if (verbose) {
 	printf ("SessionKey: ");
-	ka_PrintBytes (&token.sessionKey, sizeof(token.sessionKey));
+	ka_PrintBytes ((char *)&token.sessionKey, sizeof(token.sessionKey));
 	printf ("\nTicket (kvno = %d, len = %d): ", token.kvno, 
 		token.ticketLen);
-	ka_PrintBytes (token.ticket, token.ticketLen);
+	ka_PrintBytes ((char *)token.ticket, token.ticketLen);
 	printf ("\n");
     }
     return 0;
@@ -982,7 +990,7 @@ static GetPassword (
     ubik_ClientDestroy(lpbkConn);
     if (code) goto abort;
     printf ("Key: ");
-    ka_PrintBytes (&key, sizeof(key));
+    ka_PrintBytes ((char *)&key, sizeof(key));
     printf ("\n");
     return code;
 }
@@ -999,7 +1007,7 @@ int GetRandomKey (
     else {
 	int i;
 	printf ("Key: ");
-	ka_PrintBytes (&key, sizeof(key));
+	ka_PrintBytes ((char *)&key, sizeof(key));
 	printf (" (");
 	for (i=0; i<sizeof(key); i++) {
 	    printf ("%0.2x", ((char *)&key)[i] & 0xff);
@@ -1174,7 +1182,7 @@ int Quit (
     return 0;
 }
 
-void MyAfterProc(
+int MyAfterProc(
   struct cmd_syndesc *as)
 {
     if (!strcmp(as->name,"help")) return;
@@ -1187,7 +1195,7 @@ void MyAfterProc(
         conn = 0;
     }
 
-    return;
+    return 0;
 }
 
 int   init = 0, noauth;
@@ -1328,6 +1336,11 @@ static int MyBeforeProc(
 	  ka_StringToKey (passwd, cell, &key);
 	  code = ka_GetAdminToken (name, instance, cell, &key, KA_SIXHOURS,
 				   &token, 0/* !new */);
+	  if (code == KABADREQUEST) {
+	      des_string_to_key (passwd, &key);
+	      code = ka_GetAdminToken (name, instance, cell, &key, KA_SIXHOURS,
+				       &token, 0/* !new */);
+	  }
 	  if ((code == KABADREQUEST) && (strlen(passwd) > 8)) {
 	     /* try with only the first 8 characters incase they set
 	      * their password with an old style passwd program. */
