@@ -38,6 +38,8 @@ extern char* getenv();
 #ifdef	AFS_OSF_ENV
 extern void *malloc(int size);
 extern void *realloc(void *ptr, int size);
+#endif
+#if defined(AFS_OSF_ENV) || defined(AFS_S390_LINUX20_ENV)
 extern int PRE_Block;	/* from preempt.c */
 #else
 extern char PRE_Block;	/* from preempt.c */
@@ -56,6 +58,14 @@ extern char PRE_Block;	/* from preempt.c */
 
 #ifdef __hp9000s800
 #define MINFRAME 128
+#define STACK_ALIGN 8
+#else
+#ifdef __s390__
+#define MINFRAME    96
+#define STACK_ALIGN 8
+#else
+#define STACK_ALIGN 4
+#endif
 #endif
 
 /* Debugging macro */
@@ -269,11 +279,7 @@ int LWP_CreateProcess(ep, stacksize, priority, parm, name, pid)
 	if (stacksize < MINSTACK)
 	    stacksize = 1000;
 	else
-#ifdef __hp9000s800
-	    stacksize = 8 * ((stacksize+7) / 8);
-#else
-	    stacksize = 4 * ((stacksize+3) / 4);
-#endif
+	    stacksize = STACK_ALIGN * ((stacksize+STACK_ALIGN-1) / STACK_ALIGN);
 #ifdef	AFS_AIX32_ENV
 	if (!stackptr) {
 	    /*
@@ -337,14 +343,25 @@ int LWP_CreateProcess(ep, stacksize, priority, parm, name, pid)
 #ifdef __hp9000s800
 	savecontext(Create_Process_Part2, &temp2->context, stackptr+MINFRAME);
 #else
-#ifdef AFS_SGI62_ENV
+#if defined(AFS_SGI62_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
 	/* Need to have the sp on an 8-byte boundary for storing doubles. */
 	savecontext(Create_Process_Part2, &temp2->context,
 		    stackptr+stacksize-16); /* 16 = 2 * jmp_buf_type*/
 #else
+#if defined(AFS_SPARC64_LINUX20_ENV) || defined(AFS_SPARC_LINUX20_ENV)
+	savecontext(Create_Process_Part2, &temp2->context,
+		    stackptr+stacksize-0x40); /* lomgjmp does something
+						 with %fp + 0x38 */
+#else
+#if defined(AFS_S390_LINUX20_ENV)
+	savecontext(Create_Process_Part2, &temp2->context,
+		    stackptr+stacksize-MINFRAME);
+#else /* !AFS_S390_LINUX20_ENV */
 	savecontext(Create_Process_Part2, &temp2->context,
 		    stackptr+stacksize-sizeof(void *));
-#endif
+#endif /* AFS_S390_LINUX20_ENV */
+#endif /* AFS_SPARC64_LINUX20_ENV || AFS_SPARC_LINUX20_ENV */
+#endif /* AFS_SGI62_ENV */
 #endif
 	/* End of gross hack */
 
@@ -391,11 +408,7 @@ int LWP_CreateProcess2(ep, stacksize, priority, parm, name, pid)
 	if (stacksize < MINSTACK)
 	    stacksize = 1000;
 	else
-#ifdef __hp9000s800
-	    stacksize = 8 * ((stacksize+7) / 8);
-#else
-	    stacksize = 4 * ((stacksize+3) / 4);
-#endif
+	    stacksize = STACK_ALIGN * ((stacksize+STACK_ALIGN-1) / STACK_ALIGN);
 	if ((stackptr = (char *) malloc(stacksize)) == NULL) {
 	    Set_LWP_RC();
 	    return LWP_ENOMEM;
@@ -415,8 +428,12 @@ int LWP_CreateProcess2(ep, stacksize, priority, parm, name, pid)
 	lwp_cpptr = temp;
 #ifdef __hp9000s800
 	savecontext(Create_Process_Part2, &temp2->context, stackptr+MINFRAME);
+#else
+#if defined(AFS_S390_LINUX20_ENV)
+	savecontext(Create_Process_Part2, &temp2->context, stackptr+stacksize-MINFRAME);
 #else
 	savecontext(Create_Process_Part2, &temp2->context, stackptr+stacksize-sizeof(void *));
+#endif
 #endif
 	/* End of gross hack */
 
@@ -439,6 +456,15 @@ int LWP_CurrentProcess(pid)	/* returns pid of current process */
 	return LWP_EINIT;
 }
 
+PROCESS LWP_ThreadId()
+{
+    Debug(0, ("Entered ThreadId"))
+    if (lwp_init)
+        return lwp_cpptr;
+    else
+        return (PROCESS) 0;
+}
+
 #define LWPANCHOR (*lwp_init)
 
 int LWP_DestroyProcess(pid)		/* destroy a lightweight process */
@@ -459,13 +485,23 @@ int LWP_DestroyProcess(pid)		/* destroy a lightweight process */
 	    savecontext(Dispatcher, &(temp -> context),
 			&(LWPANCHOR.dsptchstack[MINFRAME]));
 #else
-#ifdef AFS_SGI62_ENV
+#if defined(AFS_SGI62_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
 	    savecontext(Dispatcher, &(temp -> context),
 			&(LWPANCHOR.dsptchstack[(sizeof LWPANCHOR.dsptchstack)-8]));
+#else
+#if defined(AFS_SPARC64_LINUX20_ENV) || defined(AFS_SPARC_LINUX20_ENV)
+	    savecontext(Dispatcher, &(temp -> context),
+			&(LWPANCHOR.dsptchstack[(sizeof LWPANCHOR.dsptchstack)-0x40]));
+#else
+#if defined(AFS_S390_LINUX20_ENV)
+	    savecontext(Dispatcher, &(temp -> context),
+			&(LWPANCHOR.dsptchstack[(sizeof LWPANCHOR.dsptchstack)-MINFRAME]));
 #else
 	    savecontext(Dispatcher, &(temp -> context),
 			&(LWPANCHOR.dsptchstack[(sizeof LWPANCHOR.dsptchstack)-sizeof(void *)]));
 #endif
+#endif
+#endif
 #endif
 	}
 	return LWP_SUCCESS;
@@ -914,7 +950,7 @@ static void Initialize_PCB(temp, priority, stack, stacksize, ep, parm, name)
     temp -> misc = NULL;	/* currently unused */
     temp -> next = NULL;
     temp -> prev = NULL;
-    temp -> lwp_rused = NULL;
+    temp -> lwp_rused = 0;
     temp -> level = 1;		/* non-preemptable */
 }
 
