@@ -56,11 +56,6 @@ int ROUNDS = 16;
 #endif
 
 #define XPRT_FCRYPT
-#ifdef KERNEL
-#include "../afs/permit_xprt.h"
-#else
-#include "../permit_xprt.h"
-#endif
 
 int fc_keysched (key, schedule)
   IN struct ktc_encryptionKey *key;
@@ -104,7 +99,6 @@ int fc_keysched (key, schedule)
     return 0;
 }
 
-AFS_HIDE
 afs_int32 fc_ecb_encrypt(clear, cipher, schedule, encrypt)
   IN afs_uint32  *clear;
   OUT afs_uint32 *cipher;
@@ -194,7 +188,6 @@ afs_int32 fc_ecb_encrypt(clear, cipher, schedule, encrypt)
  * NOTE: fc_cbc_encrypt now modifies its 5th argument, to permit chaining over
  * scatter/gather vectors.
  */
-AFS_HIDE
 afs_int32 fc_cbc_encrypt (input, output, length, key, xor, encrypt)
   char		*input;
   char		*output;
